﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TokenLibrary;
using Model_Syntakstræ;

namespace ParserClassLibrary
{
    public class Parser
    {
        private List<Token> input;
        private int firstPosition = 0;
        private RegionsKøreplan ast_rkp;
        private bool parseError = false;

        public Parser(List<Token> tokenInput)
        {
            this.input = tokenInput;
//            this.input.Add(new Token(Token.Tokentype.Slut, "", 0));
        }
        private Token getFirst()
        {
            //if (input.Count <= position)
            //    return new Token(Token.Tokentype.Slut, "", 0);
            return input[firstPosition];     
        }
        private void move2Next()
        {
            ++firstPosition;     
        }

        public RegionsKøreplan GetRegionskøreplan() // AST
        {
            return this.ast_rkp;
        }

        public bool parseRegionskøreplan()
        {
            // startregel
            // regionskøreplan = køreplan {køreplan} "slut"
            RegionsKøreplan rkp = new RegionsKøreplan();
            this.ast_rkp = rkp;     // foreløbig evt. fejl
            if (!parseKøreplan(rkp)) return false;

            while (getFirst().Typen != Token.Tokentype.Slut)
            {
                if (!parseKøreplan(rkp)) return false;
            }
            //this.ast_rkp = rkp;     // alt ok
            return true;
        }

        private bool parseKøreplan(RegionsKøreplan regkp)
        {
            // Køreplan = rutenr afgang {afgang}
            Køreplan kp = new Køreplan();
            //regkp.TilføjKøreplan(kp);       // hvis her evt med fejl / uden data
            if (!parseRutenr(kp)) return false;
            //regkp.TilføjKøreplan(kp);       // hvis her evt. med fejl, men rutenr er ok
            do
            {
                if (!parseAfgang(kp)) return false;
            } while (getFirst().Typen == Token.Tokentype.By);
            regkp.TilføjKøreplan(kp);       // ok
            return true;
        }

        private bool parseRutenr(Køreplan kp)
        {
            // rutenr .... nonterminal / ikke yderlig nedbrydning
            if (getFirst().Typen != Token.Tokentype.Rutenr)
            {
                Console.WriteLine("Fejl i token" + getFirst().ToString() + " : rutenr forventet");
                // evt. skip for at finde flere fejl....
                return false;
            }
            // rutenr parset ok
            string stringnr = getFirst().Leksem.Substring(1);
            kp.Nr = int.Parse(stringnr);
            move2Next();
            return true;
        }

        private bool parseAfgang(Køreplan kp)
        {
            // afgang = by tidspunkt
            Afgang afg = new Afgang();
            if (!parseBy(afg)) return false;
            if (!parseTidspunkt(afg)) return false;
            kp.TilføjAfgang(afg);       // ok
            return true;
        }

        private bool parseBy(Afgang afg)
        {
            // by .... nonterminal / ikke yderlig nedbrydning
            if (getFirst().Typen != Token.Tokentype.By)
            {
                Console.WriteLine("Fejl i token" + getFirst().ToString() + " : by forventet");
                return false;
            }
            afg.Bynavn = getFirst().Leksem;
            move2Next();
            return true;
        }

        private bool parseTidspunkt(Afgang afg)
        {
            // tidspunkt .... nonterminal / ikke yderlig nedbrydning
            if (getFirst().Typen != Token.Tokentype.Tidspunkt)
            {
                Console.WriteLine("Fejl i token" + getFirst().ToString() + " : tidspunkt forventet");
                return false;
            }
            string strTidspunkt = getFirst().Leksem;  // gem
            move2Next();                              // smid væk

            if (!tidspunktOk(strTidspunkt)) return false; // semantisk tjek

            afg.Time = int.Parse(strTidspunkt.Substring(0, 2));
            afg.Minut = int.Parse(strTidspunkt.Substring(3, 2));
            return true;
        }

        private bool tidspunktOk(string strTidspunkt)
        {
            int time = int.Parse(strTidspunkt.Substring(0,2));
            int minut = int.Parse(strTidspunkt.Substring(3, 2));
            if ((0 < time && time < 23) && (0 < minut && minut < 59))
                return true;
            Console.WriteLine("Fejl i tidspunkt " + strTidspunkt);
            return false;
        }





    }
}
